@@ -1,6 +1,6 @@
 /* Copyright (c) 2019-2022 The Khronos Group Inc.
- * Copyright (c) 2019-2022 Valve Corporation
- * Copyright (c) 2019-2022 LunarG, Inc.
+ * Copyright (c) 2019-2023 Valve Corporation
+ * Copyright (c) 2019-2023 LunarG, Inc.
  * Copyright (C) 2019-2022 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,9 +20,6 @@
  */
 #pragma once
 
-#ifndef RANGE_VECTOR_H_
-#define RANGE_VECTOR_H_
-
 #include <algorithm>
 #include <cassert>
 #include <limits>
@@ -169,6 +166,9 @@ struct split_op_keep_upper {
 
 enum class value_precedence { prefer_source, prefer_dest };
 
+template <typename Iterator, typename Map, typename Range>
+Iterator split(Iterator in, Map &map, const Range &range);
+
 // The range based sparse map implemented on the ImplMap
 template <typename Key, typename T, typename RangeKey = range<Key>, typename ImplMap = std::map<RangeKey, T>>
 class range_map {
@@ -377,7 +377,7 @@ class range_map {
                     // for the range
                     pos = split_impl(pos, bounds.end, split_op_keep_both());
                 }
-                // advance to the upper haf of the split which will be upper_bound  or to next which will both be out of bounds
+                // advance to the upper half of the split which will be upper_bound  or to next which will both be out of bounds
                 ++pos;
                 RANGE_ASSERT(!bounds.includes(pos->first.begin));
             }
@@ -387,12 +387,13 @@ class range_map {
         return range<ImplIterator>(lower, pos);
     }
 
-    ImplIterator impl_erase_range(const key_type &bounds, ImplIterator lower) {
+    template <typename TouchOp>
+    ImplIterator impl_erase_range(const key_type &bounds, ImplIterator lower, const TouchOp &touch_mapped_value) {
         // Logic assumes we are starting at a valid lower bound
         RANGE_ASSERT(!at_impl_end(lower));
         RANGE_ASSERT(lower == lower_bound_impl(bounds));
 
-        // Trim/infil the beginning if needed
+        // Trim/infill the beginning if needed
         auto current = lower;
         const auto first_begin = current->first.begin;
         if (bounds.begin > first_begin) {
@@ -411,12 +412,27 @@ class range_map {
 
         // Loop over completely contained entries and erase them
         while (!at_impl_end(current) && (current->first.end <= bounds.end)) {
-            current = impl_erase(current);
+            if (touch_mapped_value(current->second)) {
+                current = impl_erase(current);
+            } else {
+                ++current;
+            }
         }
 
         if (!at_impl_end(current) && current->first.includes(bounds.end)) {
             // last entry extends past the end of the bounds range, snip to only erase the bounded section
-            current = split_impl(current, bounds.end, split_op_keep_upper());
+            current = split_impl(current, bounds.end, split_op_keep_both());
+            // test if lower_bound (eventually) computed in split_impl is not empty.
+            // If it is not empty, then it contains values inside the bounds range,
+            // they need to be touched
+            if ((current->first & bounds).non_empty()) {
+                if (touch_mapped_value(current->second)) {
+                    current = impl_erase(current);
+                } else {
+                    // make current point to upper bound
+                    ++current;
+                }
+            }
         }
 
         RANGE_ASSERT(current == upper_bound_impl(bounds));
@@ -538,17 +554,26 @@ class range_map {
 
     iterator erase(iterator first, iterator last) { return erase(range<iterator>(first, last)); }
 
-    iterator erase_range(const key_type &bounds) {
+    // Before trying to erase a range, function touch_mapped_value is called on the mapped value.
+    // touch_mapped_value is allowed to have it's parameter type to be non const reference.
+    // If it returns true, regular erase will occur.
+    // Else, range is kept.
+    template <typename TouchOp>
+    iterator erase_range_or_touch(const key_type &bounds, const TouchOp &touch_mapped_value) {
         auto lower = lower_bound_impl(bounds);
 
         if (at_impl_end(lower) || !bounds.intersects(lower->first)) {
             // There is nothing in this range lower bound is above bound
             return iterator(lower);
         }
-        auto next = impl_erase_range(bounds, lower);
+        auto next = impl_erase_range(bounds, lower, touch_mapped_value);
         return iterator(next);
     }
 
+    iterator erase_range(const key_type &bounds) {
+        return erase_range_or_touch(bounds, [](const auto &) { return true; });
+    }
+
     void clear() { impl_map_.clear(); }
 
     iterator find(const key_type &key) { return iterator(impl_map_.find(key)); }
@@ -597,7 +622,7 @@ class range_map {
         // we don't have to check upper if just check that lower doesn't intersect (which it would if lower != upper)
         auto lower = lower_bound_impl(key);
         if (at_impl_end(lower) || !lower->first.intersects(key)) {
-            // range is not even paritally overlapped, and lower is strictly > than key
+            // range is not even partially overlapped, and lower is strictly > than key
             auto impl_insert = impl_map_.emplace_hint(lower, value);
             // auto impl_insert = impl_map_.emplace(value);
             iterator wrap_it(impl_insert);
@@ -634,6 +659,45 @@ class range_map {
         return iterator(impl_insert);
     }
 
+    // Try to insert value. If insertion failed, recursively split union of retrieved stored range with inserted range.
+    // Split at intersection of stored range and inserted range.
+    // Range intersection is merged using merge_op.
+    // Ranges before and after this intersection are recursively inserted.
+    // merge_pos should have this signature: (mapped_type& current_value, const mapped_type& new_value) -> void
+    template <typename MergeOp>
+    iterator split_and_merge_insert(const value_type &value, const MergeOp &merge_op) {
+        if (!value.first.non_empty()) {
+            return end();
+        }
+
+        if (auto [it, was_inserted] = insert(value); !was_inserted) {
+            // insert failed, so at least one stored range intersects with new range
+            const RangeKey it_range = it->first;
+            const auto &[inserted_range, insert_mapped_value] = value;
+            const RangeKey intersection = it_range & inserted_range;
+            // if intersection is empty or invalid, insertion should have succeeded
+            assert(intersection.non_empty());
+
+            const iterator split_point_it = sparse_container::split(it, *this, intersection);
+            // given it->first and instersection do intersect, split should have succeeded
+            RANGE_ASSERT(split_point_it != end());
+            // merge values at inserted range and retrieved range intersection
+            merge_op(split_point_it->second, insert_mapped_value);
+
+            // Recursively insert ranges before and after intersection
+            const RangeKey range_after_intersection(intersection.end, std::max(it_range.end, inserted_range.end));
+            const RangeKey range_before_intersection(std::min(it_range.begin, inserted_range.begin), intersection.begin);
+            split_and_merge_insert({range_after_intersection, insert_mapped_value}, merge_op);
+            if (range_before_intersection.non_empty()) {
+                return split_and_merge_insert({range_before_intersection, insert_mapped_value}, merge_op);
+            } else {
+                return split_point_it;
+            }
+        } else {
+            return it;
+        }
+    }
+
     template <typename SplitOp>
     iterator split(const iterator whole_it, const index_type &index, const SplitOp &split_op) {
         auto split_it = split_impl(whole_it.pos_, index, split_op);
@@ -650,7 +714,7 @@ class range_map {
         if (!at_impl_end(lower_impl)) {
             // If we're at end (and the hint is good, there's nothing to erase
             RANGE_ASSERT(lower == lower_bound(value.first));
-            insert_hint = impl_erase_range(value.first, lower_impl);
+            insert_hint = impl_erase_range(value.first, lower_impl, [](const auto &) { return true; });
         }
         auto inserted = impl_insert(insert_hint, std::forward<Value>(value));
         return iterator(inserted);
@@ -1533,7 +1597,7 @@ const MappedType &evaluate(const CachedLowerBound &clb, const MappedType &defaul
     return default_value;
 }
 
-// Split a range into pieces bound by the interection of the interator's range and the supplied range
+// Split a range into pieces bound by the intersection of the iterator's range and the supplied range
 template <typename Iterator, typename Map, typename Range>
 Iterator split(Iterator in, Map &map, const Range &range) {
     assert(in != map.end());  // Not designed for use with invalid iterators...
@@ -1756,14 +1820,14 @@ struct update_prefer_source {
         return false;
     }
 
-    layer_data::optional<T> insert(const T &src) const { return layer_data::optional<T>(layer_data::in_place, src); }
+    std::optional<T> insert(const T &src) const { return std::optional<T>(vvl::in_place, src); }
 };
 
 template <typename T>
 struct update_prefer_dest {
     bool update(T &dst, const T &src) const { return false; }
 
-    layer_data::optional<T> insert(const T &src) const { return layer_data::optional<T>(layer_data::in_place, src); }
+    std::optional<T> insert(const T &src) const { return std::optional<T>(vvl::in_place, src); }
 };
 
 template <typename RangeMap, typename SourceIterator = typename RangeMap::const_iterator>
@@ -1864,5 +1928,3 @@ void consolidate(RangeMap &map) {
 }
 
 }  // namespace sparse_container
-
-#endif
