@@ -1,7 +1,7 @@
-/* Copyright (c) 2019-2022 The Khronos Group Inc.
- * Copyright (c) 2019-2022 Valve Corporation
- * Copyright (c) 2019-2022 LunarG, Inc.
- * Copyright (C) 2019-2022 Google Inc.
+/* Copyright (c) 2019-2023 The Khronos Group Inc.
+ * Copyright (c) 2019-2023 Valve Corporation
+ * Copyright (c) 2019-2023 LunarG, Inc.
+ * Copyright (C) 2019-2023 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@
 #include "subresource_adapter.h"
 #include "vk_format_utils.h"
 #include <cmath>
-#include "image_state.h"
+#include "state_tracker/image_state.h"
 #include "layer_chassis_dispatch.h"
 
 namespace subresource_adapter {
@@ -69,7 +69,7 @@ uint32_t RangeEncoder::LowerBoundWithStartImpl2(VkImageAspectFlags aspect_mask,
             if (aspect_mask & aspect_bits_[0]) {
                 return 0;
             }
-            // no break
+            [[fallthrough]];
         case 1:
             if (aspect_mask & aspect_bits_[1]) {
                 return 1;
@@ -98,12 +98,12 @@ uint32_t RangeEncoder::LowerBoundWithStartImpl3(VkImageAspectFlags aspect_mask,
             if (aspect_mask & aspect_bits_[0]) {
                 return 0;
             }
-            // no break
+            [[fallthrough]];
         case 1:
             if ((aspect_mask & aspect_bits_[1])) {
                 return 1;
             }
-            // no break
+            [[fallthrough]];
         case 2:
             if ((aspect_mask & aspect_bits_[2])) {
                 return 2;
@@ -304,7 +304,7 @@ ImageRangeEncoder::ImageRangeEncoder(const IMAGE_STATE& image, const AspectParam
         for (uint32_t mip_index = 0; mip_index < limits_.mipLevel; ++mip_index) {
             subres_layers.mipLevel = mip_index;
             subres.mipLevel = mip_index;
-            auto subres_extent = image.GetSubresourceExtent(subres_layers);
+            auto subres_extent = image.GetEffectiveSubresourceExtent(subres_layers);
 
             if (linear_image_) {
                 DispatchGetImageSubresourceLayout(image.store_device_as_workaround, image.image(), &subres, &layout);
@@ -378,13 +378,6 @@ inline VkImageSubresourceRange GetRemaining(const VkImageSubresourceRange& full_
 inline bool CoversAllLayers(const VkImageSubresourceRange& full_range, VkImageSubresourceRange subres_range) {
     return (subres_range.baseArrayLayer == 0) && (subres_range.layerCount == full_range.layerCount);
 }
-inline bool CoversAllLevels(const VkImageSubresourceRange& full_range, VkImageSubresourceRange subres_range) {
-    return (subres_range.baseMipLevel == 0) && (subres_range.layerCount == full_range.levelCount);
-}
-inline bool CoversAllAspects(const VkImageSubresourceRange& full_range, VkImageSubresourceRange subres_range) {
-    return full_range.aspectMask == subres_range.aspectMask;
-}
-
 static bool SubresourceRangeIsEmpty(const VkImageSubresourceRange& range) {
     return (0 == range.aspectMask) || (0 == range.levelCount) || (0 == range.layerCount);
 }
@@ -612,8 +605,8 @@ void ImageRangeGenerator::SetUpSubresIncrementer() {
     } else if (is_3d || CoversAllLayers(full_range, subres_range_)) {
         if (!linear_image) {
             // Linear images are defined by the implementation and so we can't assume the ordering we use here
-            bool all_mips = (subres_range_.baseMipLevel == 0) && (subres_range_.levelCount == full_range.levelCount);
-            bool all_aspects = subres_range_.aspectMask == full_range.aspectMask;
+            const bool all_mips = (subres_range_.baseMipLevel == 0) && (subres_range_.levelCount == full_range.levelCount);
+            const bool all_aspects = subres_range_.aspectMask == full_range.aspectMask;
             if (all_aspects && all_mips) {
                 set_initial_pos_fn_ = &ImageRangeGenerator::SetInitialPosAllSubres;
             } else {
@@ -854,4 +847,4 @@ void ImageRangeGenerator::IncrementerState::Set(uint32_t y_count_, uint32_t laye
     incr_layer_z = z_step;
 }
 
-};  // namespace subresource_adapter
+}  // namespace subresource_adapter
