@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2021-2022 The Khronos Group Inc.
+# Copyright (c) 2021-2023 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Author: Spencer Fricke <s.fricke@samsung.com>
 
 import os,re,sys,string,json
 import xml.etree.ElementTree as etree
@@ -143,7 +141,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
         copyright += '\n'
         copyright += '/***************************************************************************\n'
         copyright += ' *\n'
-        copyright += ' * Copyright (c) 2021-2022 The Khronos Group Inc.\n'
+        copyright += ' * Copyright (c) 2021-2023 The Khronos Group Inc.\n'
         copyright += ' *\n'
         copyright += ' * Licensed under the Apache License, Version 2.0 (the "License");\n'
         copyright += ' * you may not use this file except in compliance with the License.\n'
@@ -156,15 +154,12 @@ class CommandValidationOutputGenerator(OutputGenerator):
         copyright += ' * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n'
         copyright += ' * See the License for the specific language governing permissions and\n'
         copyright += ' * limitations under the License.\n'
-        copyright += ' *\n'
-        copyright += ' * Author: Spencer Fricke <s.fricke@samsung.com>\n'
-        copyright += ' *\n'
         copyright += ' ****************************************************************************/\n'
         write(copyright, file=self.outFile)
 
         if self.source_file:
             write('#include "vk_layer_logging.h"', file=self.outFile)
-            write('#include "core_validation.h"', file=self.outFile)
+            write('#include "core_checks/core_validation.h"', file=self.outFile)
         elif self.header_file:
             write('#pragma once', file=self.outFile)
             write('#include <array>', file=self.outFile)
@@ -181,6 +176,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
             write(self.commandRecordingList(), file=self.outFile)
             write(self.commandQueueTypeList(), file=self.outFile)
             write(self.commandRenderPassList(), file=self.outFile)
+            write(self.commandVideoCodingList(), file=self.outFile)
             write(self.commandBufferLevelList(), file=self.outFile)
             write(self.validateFunction(), file=self.outFile)
             write(self.dynamicFunction(), file=self.outFile)
@@ -263,7 +259,8 @@ typedef enum CBDynamicStatus {\n'''
 } CBDynamicStatus;
 
 using CBDynamicFlags = std::bitset<CB_DYNAMIC_STATUS_NUM>;
-std::string DynamicStateString(CBDynamicFlags const &dynamic_state);
+const char* DynamicStateToString(CBDynamicStatus status);
+std::string DynamicStatesToString(CBDynamicFlags const &dynamic_state);
 struct VkPipelineDynamicStateCreateInfo;
 CBDynamicFlags MakeStaticStateMask(VkPipelineDynamicStateCreateInfo const *info);
 '''
@@ -371,6 +368,48 @@ static const std::array<CommandSupportedRenderPass, CMD_RANGE_SIZE> kGeneratedRe
         output += '}};'
         return output
 
+    #
+    # For each CMD_TYPE give a videocoding restriction and a *-videocoding VUID
+    def commandVideoCodingList(self):
+        output = '''
+enum CMD_VIDEO_CODING_TYPE {
+    CMD_VIDEO_CODING_INSIDE,
+    CMD_VIDEO_CODING_OUTSIDE,
+    CMD_VIDEO_CODING_BOTH
+};
+struct CommandSupportedVideoCoding {
+    CMD_VIDEO_CODING_TYPE videoCoding;
+    const char* vuid;
+};
+static const std::array<CommandSupportedVideoCoding, CMD_RANGE_SIZE> kGeneratedVideoCodingList = {{
+    {CMD_VIDEO_CODING_BOTH, kVUIDUndefined}, // CMD_NONE\n'''
+        for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
+            video_coding_type = ''
+            video_coding = cmdinfo.elem.attrib.get('videocoding')
+            if video_coding is None:
+                video_coding = 'outside'
+            if video_coding == 'inside':
+                video_coding_type = 'CMD_VIDEO_CODING_INSIDE'
+            elif video_coding == 'outside':
+                video_coding_type = 'CMD_VIDEO_CODING_OUTSIDE'
+            elif video_coding != 'both':
+                print("videocoding attribute was %s and not known, need to update generation code", video_coding)
+                sys.exit(1)
+
+            # Only will be a VUID if not BOTH
+            if video_coding == 'both':
+                output += '    {CMD_VIDEO_CODING_BOTH, kVUIDUndefined},\n'
+            else:
+                vuid = 'VUID-' + name + '-videocoding'
+                if vuid not in self.valid_vuids:
+                    print("Warning: Could not find {} in validusage.json".format(vuid))
+                    vuid = vuid.replace('VUID-', 'UNASSIGNED-')
+                output += '    {' + video_coding_type + ', \"' + vuid + '\"},\n'
+        output += '}};'
+        return output
+
     #
     # For each CMD_TYPE give a buffer level restriction and add a *-bufferlevel VUID
     def commandBufferLevelList(self):
@@ -437,6 +476,14 @@ bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE &cb_state, const CMD_TYPE cm
         skip |= InsideRenderPass(cb_state, caller_name, supportedRenderPass.vuid);
     }
 
+    // Validate if command is inside or outside a video coding scope if applicable
+    const auto supportedVideoCoding = kGeneratedVideoCodingList[cmd];
+    if (supportedVideoCoding.videoCoding == CMD_VIDEO_CODING_INSIDE) {
+        skip |= OutsideVideoCodingScope(cb_state, caller_name, supportedVideoCoding.vuid);
+    } else if (supportedVideoCoding.videoCoding == CMD_VIDEO_CODING_OUTSIDE) {
+        skip |= InsideVideoCodingScope(cb_state, caller_name, supportedVideoCoding.vuid);
+    }
+
     // Validate if command has to be recorded in a primary command buffer
     const auto supportedBufferLevel = kGeneratedBufferLevelList[cmd];
     if (supportedBufferLevel != nullptr) {
@@ -476,7 +523,11 @@ static CBDynamicStatus ConvertToCBDynamicStatus(VkDynamicState state) {
 '''
 
         output += '''
-std::string DynamicStateString(CBDynamicFlags const &dynamic_state) {
+const char* DynamicStateToString(CBDynamicStatus status) {
+    return string_VkDynamicState(ConvertToDynamicState(status));
+}
+
+std::string DynamicStatesToString(CBDynamicFlags const &dynamic_state) {
     std::string ret;
     // enum is not zero based
     for (int index = 1; index < CB_DYNAMIC_STATUS_NUM; ++index) {
