@@ -1,41 +1,24 @@
 /*
- * Copyright (c) 2015-2022 The Khronos Group Inc.
- * Copyright (c) 2015-2022 Valve Corporation
- * Copyright (c) 2015-2022 LunarG, Inc.
- * Copyright (c) 2015-2022 Google, Inc.
+ * Copyright (c) 2015-2023 The Khronos Group Inc.
+ * Copyright (c) 2015-2023 Valve Corporation
+ * Copyright (c) 2015-2023 LunarG, Inc.
+ * Copyright (c) 2015-2023 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Author: Chia-I Wu <olvaffe@gmail.com>
- * Author: Chris Forbes <chrisf@ijw.co.nz>
- * Author: Courtney Goeltzenleuchter <courtney@LunarG.com>
- * Author: Mark Lobodzinski <mark@lunarg.com>
- * Author: Mike Stroyan <mike@LunarG.com>
- * Author: Tobin Ehlis <tobine@google.com>
- * Author: Tony Barbour <tony@LunarG.com>
- * Author: Cody Northrop <cnorthrop@google.com>
- * Author: Dave Houlton <daveh@lunarg.com>
- * Author: Jeremy Kniager <jeremyk@lunarg.com>
- * Author: Shannon McPherson <shannon@lunarg.com>
- * Author: John Zulauf <jzulauf@lunarg.com>
  */
 
-#include "../layer_validation_tests.h"
+#include "../framework/layer_validation_tests.h"
 #include "vk_extension_helper.h"
 
 #include <algorithm>
 #include <array>
 #include <chrono>
-#include <memory>
-#include <mutex>
 #include <thread>
 
-#include "cast_utils.h"
-
 //
 // POSITIVE VALIDATION TESTS
 //
@@ -80,15 +63,13 @@ TEST_F(VkPositiveLayerTest, DeleteDescriptorSetLayoutsBeforeDescriptorSets) {
     ds_type_count.type = VK_DESCRIPTOR_TYPE_SAMPLER;
     ds_type_count.descriptorCount = 1;
 
-    VkDescriptorPoolCreateInfo ds_pool_ci = LvlInitStruct<VkDescriptorPoolCreateInfo>();
+    auto ds_pool_ci = LvlInitStruct<VkDescriptorPoolCreateInfo>();
     ds_pool_ci.flags = VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT;
     ds_pool_ci.maxSets = 1;
     ds_pool_ci.poolSizeCount = 1;
     ds_pool_ci.pPoolSizes = &ds_type_count;
 
-    VkDescriptorPool ds_pool_one;
-    err = vk::CreateDescriptorPool(m_device->device(), &ds_pool_ci, NULL, &ds_pool_one);
-    ASSERT_VK_SUCCESS(err);
+    vk_testing::DescriptorPool ds_pool_one(*m_device, ds_pool_ci);
 
     VkDescriptorSetLayoutBinding dsl_binding = {};
     dsl_binding.binding = 0;
@@ -101,16 +82,14 @@ TEST_F(VkPositiveLayerTest, DeleteDescriptorSetLayoutsBeforeDescriptorSets) {
     {
         const VkDescriptorSetLayoutObj ds_layout(m_device, {dsl_binding});
 
-        VkDescriptorSetAllocateInfo alloc_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
+        auto alloc_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
         alloc_info.descriptorSetCount = 1;
-        alloc_info.descriptorPool = ds_pool_one;
+        alloc_info.descriptorPool = ds_pool_one.handle();
         alloc_info.pSetLayouts = &ds_layout.handle();
         err = vk::AllocateDescriptorSets(m_device->device(), &alloc_info, &descriptorSet);
         ASSERT_VK_SUCCESS(err);
     }  // ds_layout destroyed
-    err = vk::FreeDescriptorSets(m_device->device(), ds_pool_one, 1, &descriptorSet);
-
-    vk::DestroyDescriptorPool(m_device->device(), ds_pool_one, NULL);
+    vk::FreeDescriptorSets(m_device->device(), ds_pool_one.handle(), 1, &descriptorSet);
 }
 
 // This is a positive test. No failures are expected.
@@ -147,7 +126,7 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
         image_info.imageView = view;
         image_info.imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
 
-        VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+        auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
         descriptor_write.dstSet = descriptor_set.set_;
         descriptor_write.dstBinding = 0;
         descriptor_write.descriptorCount = 1;
@@ -169,7 +148,7 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
     // Buffer Case
     {
         uint32_t queue_family_index = 0;
-        VkBufferCreateInfo buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
+        auto buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
         buffer_create_info.size = 1024;
         buffer_create_info.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
         buffer_create_info.queueFamilyIndexCount = 1;
@@ -187,7 +166,7 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
         buffer_info.offset = 0;
         buffer_info.range = 1024;
 
-        VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+        auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
         descriptor_write.dstSet = descriptor_set.set_;
         descriptor_write.dstBinding = 0;
         descriptor_write.descriptorCount = 1;
@@ -209,7 +188,7 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
     // Texel Buffer Case
     {
         uint32_t queue_family_index = 0;
-        VkBufferCreateInfo buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
+        auto buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
         buffer_create_info.size = 1024;
         buffer_create_info.usage = VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT;
         buffer_create_info.queueFamilyIndexCount = 1;
@@ -218,24 +197,23 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
         VkBufferObj buffer;
         buffer.init(*m_device, buffer_create_info);
 
-        VkBufferViewCreateInfo buff_view_ci = LvlInitStruct<VkBufferViewCreateInfo>();
+        auto buff_view_ci = LvlInitStruct<VkBufferViewCreateInfo>();
         buff_view_ci.buffer = buffer.handle();
         buff_view_ci.format = format_texel_case;
         buff_view_ci.range = VK_WHOLE_SIZE;
-        VkBufferView buffer_view;
-        VkResult err = vk::CreateBufferView(m_device->device(), &buff_view_ci, NULL, &buffer_view);
-        ASSERT_VK_SUCCESS(err);
+        vk_testing::BufferView buffer_view(*m_device, buff_view_ci);
+
         OneOffDescriptorSet descriptor_set(m_device,
                                            {
                                                {0, VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr},
                                            });
 
-        VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+        auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
         descriptor_write.dstSet = descriptor_set.set_;
         descriptor_write.dstBinding = 0;
         descriptor_write.descriptorCount = 1;
         descriptor_write.descriptorType = VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER;
-        descriptor_write.pTexelBufferView = &buffer_view;
+        descriptor_write.pTexelBufferView = &buffer_view.handle();
 
         // Set pImageInfo and pBufferInfo to invalid values, which should be
         //  ignored for descriptorType ==
@@ -247,8 +225,6 @@ TEST_F(VkPositiveLayerTest, IgnoreUnrelatedDescriptor) {
         descriptor_write.pBufferInfo = reinterpret_cast<const VkDescriptorBufferInfo *>(invalid_ptr);
 
         vk::UpdateDescriptorSets(m_device->device(), 1, &descriptor_write, 0, NULL);
-
-        vk::DestroyBufferView(m_device->device(), buffer_view, NULL);
     }
 }
 
@@ -262,10 +238,7 @@ TEST_F(VkPositiveLayerTest, ImmutableSamplerOnlyDescriptor) {
                                                      {0, VK_DESCRIPTOR_TYPE_SAMPLER, 1, VK_SHADER_STAGE_FRAGMENT_BIT, nullptr},
                                                  });
 
-    VkSamplerCreateInfo sampler_ci = SafeSaneSamplerCreateInfo();
-    VkSampler sampler;
-    VkResult err = vk::CreateSampler(m_device->device(), &sampler_ci, NULL, &sampler);
-    ASSERT_VK_SUCCESS(err);
+    vk_testing::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
 
     const VkPipelineLayoutObj pipeline_layout(m_device, {&descriptor_set.layout_});
 
@@ -275,7 +248,7 @@ TEST_F(VkPositiveLayerTest, ImmutableSamplerOnlyDescriptor) {
     vk::CmdBindDescriptorSets(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline_layout.handle(), 0, 1,
                               &descriptor_set.set_, 0, nullptr);
 
-    vk::DestroySampler(m_device->device(), sampler, NULL);
+    sampler.destroy();
 
     m_commandBuffer->EndRenderPass();
     m_commandBuffer->end();
@@ -284,7 +257,6 @@ TEST_F(VkPositiveLayerTest, ImmutableSamplerOnlyDescriptor) {
 // This is a positive test. No failures are expected.
 TEST_F(VkPositiveLayerTest, EmptyDescriptorUpdateTest) {
     TEST_DESCRIPTION("Update last descriptor in a set that includes an empty binding");
-    VkResult err;
 
     ASSERT_NO_FATAL_FAILURE(Init());
     if (IsPlatform(kNexusPlayer)) {
@@ -299,42 +271,18 @@ TEST_F(VkPositiveLayerTest, EmptyDescriptorUpdateTest) {
                                      });
 
     // Create a buffer to be used for update
-    VkBufferCreateInfo buff_ci = LvlInitStruct<VkBufferCreateInfo>();
+    auto buff_ci = LvlInitStruct<VkBufferCreateInfo>();
     buff_ci.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
     buff_ci.size = 256;
     buff_ci.sharingMode = VK_SHARING_MODE_EXCLUSIVE;
-    VkBuffer buffer;
-    err = vk::CreateBuffer(m_device->device(), &buff_ci, NULL, &buffer);
-    ASSERT_VK_SUCCESS(err);
-    // Have to bind memory to buffer before descriptor update
-    VkMemoryAllocateInfo mem_alloc = LvlInitStruct<VkMemoryAllocateInfo>();
-    mem_alloc.allocationSize = 512;  // one allocation for both buffers
-    mem_alloc.memoryTypeIndex = 0;
-
-    VkMemoryRequirements mem_reqs;
-    vk::GetBufferMemoryRequirements(m_device->device(), buffer, &mem_reqs);
-    bool pass = m_device->phy().set_memory_type(mem_reqs.memoryTypeBits, &mem_alloc, 0);
-    if (!pass) {
-        vk::DestroyBuffer(m_device->device(), buffer, NULL);
-        GTEST_SKIP() << "Failed to allocate memory";
-    }
-    // Make sure allocation is sufficiently large to accommodate buffer requirements
-    if (mem_reqs.size > mem_alloc.allocationSize) {
-        mem_alloc.allocationSize = mem_reqs.size;
-    }
-
-    VkDeviceMemory mem;
-    err = vk::AllocateMemory(m_device->device(), &mem_alloc, NULL, &mem);
-    ASSERT_VK_SUCCESS(err);
-    err = vk::BindBufferMemory(m_device->device(), buffer, mem, 0);
-    ASSERT_VK_SUCCESS(err);
+    vk_testing::Buffer buffer(*m_device, buff_ci);
 
     // Only update the descriptor at binding 2
     VkDescriptorBufferInfo buff_info = {};
-    buff_info.buffer = buffer;
+    buff_info.buffer = buffer.handle();
     buff_info.offset = 0;
     buff_info.range = VK_WHOLE_SIZE;
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstBinding = 2;
     descriptor_write.descriptorCount = 1;
     descriptor_write.pTexelBufferView = nullptr;
@@ -344,10 +292,6 @@ TEST_F(VkPositiveLayerTest, EmptyDescriptorUpdateTest) {
     descriptor_write.dstSet = ds.set_;
 
     vk::UpdateDescriptorSets(m_device->device(), 1, &descriptor_write, 0, NULL);
-
-    // Cleanup
-    vk::FreeMemory(m_device->device(), mem, NULL);
-    vk::DestroyBuffer(m_device->device(), buffer, NULL);
 }
 
 // This is a positive test. No failures are expected.
@@ -363,7 +307,7 @@ TEST_F(VkPositiveLayerTest, PushDescriptorNullDstSetTest) {
 
     ASSERT_NO_FATAL_FAILURE(InitState());
 
-    VkPhysicalDevicePushDescriptorPropertiesKHR push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
+    auto push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
     GetPhysicalDeviceProperties2(push_descriptor_prop);
     if (push_descriptor_prop.maxPushDescriptors < 1) {
         // Some implementations report an invalid maxPushDescriptors of 0
@@ -398,7 +342,7 @@ TEST_F(VkPositiveLayerTest, PushDescriptorNullDstSetTest) {
     buff_info.buffer = vbo.handle();
     buff_info.offset = 0;
     buff_info.range = sizeof(vbo_data);
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstBinding = 2;
     descriptor_write.descriptorCount = 1;
     descriptor_write.pTexelBufferView = nullptr;
@@ -432,7 +376,7 @@ TEST_F(VkPositiveLayerTest, PushDescriptorUnboundSetTest) {
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
 
-    VkPhysicalDevicePushDescriptorPropertiesKHR push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
+    auto push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
     GetPhysicalDeviceProperties2(push_descriptor_prop);
     if (push_descriptor_prop.maxPushDescriptors < 1) {
         // Some implementations report an invalid maxPushDescriptors of 0
@@ -526,8 +470,7 @@ TEST_F(VkPositiveLayerTest, BindingPartiallyBound) {
     ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
 
     VkDescriptorBindingFlagsEXT ds_binding_flags[2] = {};
-    VkDescriptorSetLayoutBindingFlagsCreateInfoEXT layout_createinfo_binding_flags =
-        LvlInitStruct<VkDescriptorSetLayoutBindingFlagsCreateInfoEXT>();
+    auto layout_createinfo_binding_flags = LvlInitStruct<VkDescriptorSetLayoutBindingFlagsCreateInfoEXT>();
     ds_binding_flags[0] = 0;
     // No Error
     ds_binding_flags[1] = VK_DESCRIPTOR_BINDING_PARTIALLY_BOUND_BIT_EXT;
@@ -546,7 +489,7 @@ TEST_F(VkPositiveLayerTest, BindingPartiallyBound) {
                                        0, &layout_createinfo_binding_flags, 0);
     const VkPipelineLayoutObj pipeline_layout(m_device, {&descriptor_set.layout_});
     uint32_t qfi = 0;
-    VkBufferCreateInfo buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
+    auto buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
     buffer_create_info.size = 32;
     buffer_create_info.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
     buffer_create_info.queueFamilyIndexCount = 1;
@@ -560,7 +503,7 @@ TEST_F(VkPositiveLayerTest, BindingPartiallyBound) {
     buffer_info[0].offset = 0;
     buffer_info[0].range = sizeof(uint32_t);
 
-    VkBufferCreateInfo index_buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
+    auto index_buffer_create_info = LvlInitStruct<VkBufferCreateInfo>();
     index_buffer_create_info.size = sizeof(uint32_t);
     index_buffer_create_info.usage = VK_BUFFER_USAGE_INDEX_BUFFER_BIT;
     VkBufferObj index_buffer;
@@ -595,7 +538,7 @@ TEST_F(VkPositiveLayerTest, BindingPartiallyBound) {
     pipe.AddShader(&fs);
     pipe.AddDefaultColorAttachment();
     pipe.CreateVKPipeline(pipeline_layout.handle(), m_renderPass);
-    VkCommandBufferBeginInfo begin_info = LvlInitStruct<VkCommandBufferBeginInfo>();
+    auto begin_info = LvlInitStruct<VkCommandBufferBeginInfo>();
     m_commandBuffer->begin(&begin_info);
     vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.handle());
     m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
@@ -624,7 +567,7 @@ TEST_F(VkPositiveLayerTest, PushDescriptorSetUpdatingSetNumber) {
         GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
-    VkPhysicalDevicePushDescriptorPropertiesKHR push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
+    auto push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
     GetPhysicalDeviceProperties2(push_descriptor_prop);
     if (push_descriptor_prop.maxPushDescriptors < 1) {
         // Some implementations report an invalid maxPushDescriptors of 0
@@ -756,7 +699,7 @@ TEST_F(VkPositiveLayerTest, DynamicOffsetWithInactiveBinding) {
     // Create two buffers to update the descriptors with
     // The first will be 2k and used for bindings 0 & 1, the second is 1k for binding 2
     uint32_t qfi = 0;
-    VkBufferCreateInfo buffCI = LvlInitStruct<VkBufferCreateInfo>();
+    auto buffCI = LvlInitStruct<VkBufferCreateInfo>();
     buffCI.size = 2048;
     buffCI.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
     buffCI.queueFamilyIndexCount = 1;
@@ -780,7 +723,7 @@ TEST_F(VkPositiveLayerTest, DynamicOffsetWithInactiveBinding) {
     buff_info[2].offset = 0;
     buff_info[2].range = 512;
 
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstSet = descriptor_set.set_;
     descriptor_write.dstBinding = 0;
     descriptor_write.descriptorCount = BINDING_COUNT;
@@ -836,7 +779,7 @@ TEST_F(VkPositiveLayerTest, CreateDescriptorSetBindingWithIgnoredSamplers) {
 
     // In addition to the extension being supported we need to have at least one available
     // Some implementations report an invalid maxPushDescriptors of 0
-    VkPhysicalDevicePushDescriptorPropertiesKHR push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
+    auto push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
     GetPhysicalDeviceProperties2(push_descriptor_prop);
     bool push_descriptor_found = push_descriptor_prop.maxPushDescriptors > 0;
 
@@ -860,12 +803,9 @@ TEST_F(VkPositiveLayerTest, CreateDescriptorSetBindingWithIgnoredSamplers) {
             {7, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_FRAGMENT_BIT, hopefully_undereferencable_pointer},
             {8, VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT, 1, VK_SHADER_STAGE_FRAGMENT_BIT, hopefully_undereferencable_pointer},
         };
-        const VkDescriptorSetLayoutCreateInfo dslci = {VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO, nullptr, 0,
-                                                       static_cast<uint32_t>(size(non_sampler_bindings)), non_sampler_bindings};
-        VkDescriptorSetLayout dsl;
-        const VkResult err = vk::CreateDescriptorSetLayout(m_device->device(), &dslci, nullptr, &dsl);
-        ASSERT_VK_SUCCESS(err);
-        vk::DestroyDescriptorSetLayout(m_device->device(), dsl, nullptr);
+        const auto dslci =
+            LvlInitStruct<VkDescriptorSetLayoutCreateInfo>(nullptr, 0u, size32(non_sampler_bindings), non_sampler_bindings);
+        vk_testing::DescriptorSetLayout dsl(*m_device, dslci);
     }
 
     if (push_descriptor_found) {
@@ -880,13 +820,10 @@ TEST_F(VkPositiveLayerTest, CreateDescriptorSetBindingWithIgnoredSamplers) {
                 {5, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_FRAGMENT_BIT, hopefully_undereferencable_pointer},
                 {6, VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT, 1, VK_SHADER_STAGE_FRAGMENT_BIT, hopefully_undereferencable_pointer},
             };
-            const VkDescriptorSetLayoutCreateInfo dslci = {VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO, nullptr,
-                                                           VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR,
-                                                           static_cast<uint32_t>(size(non_sampler_bindings)), non_sampler_bindings};
-            VkDescriptorSetLayout dsl;
-            const VkResult err = vk::CreateDescriptorSetLayout(m_device->device(), &dslci, nullptr, &dsl);
-            ASSERT_VK_SUCCESS(err);
-            vk::DestroyDescriptorSetLayout(m_device->device(), dsl, nullptr);
+            const auto dslci = LvlInitStruct<VkDescriptorSetLayoutCreateInfo>(
+                nullptr, static_cast<VkDescriptorSetLayoutCreateFlags>(VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR),
+                size32(non_sampler_bindings), non_sampler_bindings);
+            vk_testing::DescriptorSetLayout dsl(*m_device, dslci);
         }
     }
 }
@@ -901,7 +838,7 @@ TEST_F(VkPositiveLayerTest, PushingDescriptorSetWithImmutableSampler) {
         GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
 
-    VkPhysicalDevicePushDescriptorPropertiesKHR push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
+    auto push_descriptor_prop = LvlInitStruct<VkPhysicalDevicePushDescriptorPropertiesKHR>();
     GetPhysicalDeviceProperties2(push_descriptor_prop);
     if (push_descriptor_prop.maxPushDescriptors < 1) {
         // Some implementations report an invalid maxPushDescriptors of 0
@@ -935,7 +872,7 @@ TEST_F(VkPositiveLayerTest, PushingDescriptorSetWithImmutableSampler) {
     img_info.imageView = imageView;
     img_info.imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
 
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstBinding = 0;
     descriptor_write.descriptorCount = 1;
     descriptor_write.pTexelBufferView = nullptr;
@@ -1020,7 +957,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
     mutable_descriptor_type_lists[1].descriptorTypeCount = 0;
     mutable_descriptor_type_lists[1].pDescriptorTypes = nullptr;
 
-    VkMutableDescriptorTypeCreateInfoEXT mdtci = LvlInitStruct<VkMutableDescriptorTypeCreateInfoEXT>();
+    auto mdtci = LvlInitStruct<VkMutableDescriptorTypeCreateInfoEXT>();
     mdtci.mutableDescriptorTypeListCount = 2;
     mdtci.pMutableDescriptorTypeLists = mutable_descriptor_type_lists;
 
@@ -1030,7 +967,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
     pool_sizes[1].type = VK_DESCRIPTOR_TYPE_MUTABLE_EXT;
     pool_sizes[1].descriptorCount = 2;
 
-    VkDescriptorPoolCreateInfo ds_pool_ci = LvlInitStruct<VkDescriptorPoolCreateInfo>(&mdtci);
+    auto ds_pool_ci = LvlInitStruct<VkDescriptorPoolCreateInfo>(&mdtci);
     ds_pool_ci.maxSets = 2;
     ds_pool_ci.poolSizeCount = 2;
     ds_pool_ci.pPoolSizes = pool_sizes;
@@ -1050,7 +987,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
     bindings[1].stageFlags = VK_SHADER_STAGE_ALL;
     bindings[1].pImmutableSamplers = nullptr;
 
-    VkDescriptorSetLayoutCreateInfo create_info = LvlInitStruct<VkDescriptorSetLayoutCreateInfo>(&mdtci);
+    auto create_info = LvlInitStruct<VkDescriptorSetLayoutCreateInfo>(&mdtci);
     create_info.bindingCount = 2;
     create_info.pBindings = bindings;
 
@@ -1060,7 +997,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
 
     VkDescriptorSetLayout layouts[2] = {set_layout_handle, set_layout_handle};
 
-    VkDescriptorSetAllocateInfo allocate_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
+    auto allocate_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
     allocate_info.descriptorPool = pool.handle();
     allocate_info.descriptorSetCount = 2;
     allocate_info.pSetLayouts = layouts;
@@ -1068,7 +1005,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
     VkDescriptorSet descriptor_sets[2];
     vk::AllocateDescriptorSets(device(), &allocate_info, descriptor_sets);
 
-    VkBufferCreateInfo buffer_ci = LvlInitStruct<VkBufferCreateInfo>();
+    auto buffer_ci = LvlInitStruct<VkBufferCreateInfo>();
     buffer_ci.size = 32;
     buffer_ci.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT;
 
@@ -1080,7 +1017,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
     buffer_info.offset = 0;
     buffer_info.range = buffer_ci.size;
 
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstSet = descriptor_sets[0];
     descriptor_write.dstBinding = 0;
     descriptor_write.descriptorCount = 1;
@@ -1089,7 +1026,7 @@ TEST_F(VkPositiveLayerTest, CopyMutableDescriptors) {
 
     vk::UpdateDescriptorSets(m_device->device(), 1, &descriptor_write, 0, nullptr);
 
-    VkCopyDescriptorSet copy_set = LvlInitStruct<VkCopyDescriptorSet>();
+    auto copy_set = LvlInitStruct<VkCopyDescriptorSet>();
     copy_set.srcSet = descriptor_sets[0];
     copy_set.srcBinding = 0;
     copy_set.dstSet = descriptor_sets[1];
@@ -1127,7 +1064,7 @@ TEST_F(VkPositiveLayerTest, TestImageViewAsDescriptorReadAndInputAttachment) {
     attach_desc.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
     attach_desc.finalLayout = VK_IMAGE_LAYOUT_GENERAL;
 
-    VkRenderPassCreateInfo rpci = LvlInitStruct<VkRenderPassCreateInfo>();
+    auto rpci = LvlInitStruct<VkRenderPassCreateInfo>();
     rpci.subpassCount = 1;
     rpci.pSubpasses = &subpass;
     rpci.attachmentCount = 1;
@@ -1178,7 +1115,7 @@ TEST_F(VkPositiveLayerTest, TestImageViewAsDescriptorReadAndInputAttachment) {
 
     vk_testing::Framebuffer framebuffer(*m_device, fbci);
 
-    VkRenderPassBeginInfo rpbi = LvlInitStruct<VkRenderPassBeginInfo>();
+    auto rpbi = LvlInitStruct<VkRenderPassBeginInfo>();
     rpbi.framebuffer = framebuffer.handle();
     rpbi.renderPass = render_pass.handle();
     rpbi.renderArea.extent.width = width;
@@ -1234,7 +1171,7 @@ TEST_F(VkPositiveLayerTest, TestImageViewAsDescriptorReadAndInputAttachment) {
     image_info.sampler = sampler.handle();
     image_info.imageView = image_view.handle();
     image_info.imageLayout = VK_IMAGE_LAYOUT_GENERAL;
-    VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+    auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
     descriptor_write.dstSet = descriptor_set.set_;
     descriptor_write.dstBinding = 0;
     descriptor_write.descriptorCount = 1;
@@ -1271,10 +1208,10 @@ TEST_F(VkPositiveLayerTest, UpdateImageDescriptorSetThatHasImageViewUsage) {
     VkImageObj image(m_device);
     image.Init(32, 32, 1, VK_FORMAT_B8G8R8A8_UNORM, VK_IMAGE_USAGE_SAMPLED_BIT, VK_IMAGE_TILING_OPTIMAL, 0);
 
-    VkImageViewUsageCreateInfo *image_view_usage_ci = new VkImageViewUsageCreateInfo();
-    *image_view_usage_ci = LvlInitStruct<VkImageViewUsageCreateInfo>();
-    image_view_usage_ci->usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    auto image_view_ci = LvlInitStruct<VkImageViewCreateInfo>(image_view_usage_ci);
+    auto image_view_usage_ci = LvlInitStruct<VkImageViewUsageCreateInfo>();
+    image_view_usage_ci.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
+
+    auto image_view_ci = LvlInitStruct<VkImageViewCreateInfo>(&image_view_usage_ci);
     image_view_ci.image = image.handle();
     image_view_ci.viewType = VK_IMAGE_VIEW_TYPE_2D;
     image_view_ci.format = VK_FORMAT_B8G8R8A8_UNORM;
@@ -1287,8 +1224,6 @@ TEST_F(VkPositiveLayerTest, UpdateImageDescriptorSetThatHasImageViewUsage) {
     vk_testing::ImageView image_view;
     image_view.init(*m_device, image_view_ci);
 
-    delete image_view_usage_ci;
-
     VkSamplerCreateInfo sampler_ci = SafeSaneSamplerCreateInfo();
     vk_testing::Sampler sampler;
     sampler.init(*m_device, sampler_ci);
@@ -1342,7 +1277,7 @@ TEST_F(VkPositiveLayerTest, MultipleThreadsUsingHostOnlyDescriptorSet) {
         image_info.imageView = view1;
         image_info.imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
 
-        VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+        auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
         descriptor_write.dstSet = descriptor_set.set_;
         descriptor_write.dstBinding = 0;
         descriptor_write.descriptorCount = 1;
@@ -1356,7 +1291,7 @@ TEST_F(VkPositiveLayerTest, MultipleThreadsUsingHostOnlyDescriptorSet) {
         image_info.imageView = view2;
         image_info.imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
 
-        VkWriteDescriptorSet descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+        auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
         descriptor_write.dstSet = descriptor_set.set_;
         descriptor_write.dstBinding = 0;
         descriptor_write.dstArrayElement = 1;
@@ -1370,3 +1305,243 @@ TEST_F(VkPositiveLayerTest, MultipleThreadsUsingHostOnlyDescriptorSet) {
     std::array<std::thread, 2> threads = {std::thread(testing_thread1), std::thread(testing_thread2)};
     for (auto &t : threads) t.join();
 }
+
+TEST_F(VkPositiveLayerTest, BindingEmptyDescriptorSets) {
+    ASSERT_NO_FATAL_FAILURE(Init());
+
+    OneOffDescriptorSet empty_ds(m_device, {});
+    const VkPipelineLayoutObj pipeline_layout(m_device, {&empty_ds.layout_});
+
+    m_commandBuffer->begin();
+    vk::CmdBindDescriptorSets(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline_layout.handle(), 0, 1,
+                              &empty_ds.set_, 0, nullptr);
+    m_commandBuffer->end();
+}
+
+TEST_F(VkPositiveLayerTest, DrawingWithUnboundUnusedSetWithInputAttachments) {
+    TEST_DESCRIPTION(
+        "Test issuing draw command with pipeline layout that has 2 descriptor sets with input attachment descriptors. "
+        "The second descriptor set is unused and unbound. Its purpose is to catch regression of the following bug or similar "
+        "issues when accessing unbound set: https://github.com/KhronosGroup/Vulkan-ValidationLayers/pull/4576");
+
+    ASSERT_NO_FATAL_FAILURE(Init());
+    ASSERT_NO_FATAL_FAILURE(InitViewport());
+    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
+
+    const auto width = static_cast<uint32_t>(m_viewports[0].width);
+    const auto height = static_cast<uint32_t>(m_viewports[0].height);
+    const VkFormat format = VK_FORMAT_R8G8B8A8_UNORM;
+    const VkImageUsageFlags usage = VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT;
+
+    VkImageObj image_input(m_device);
+    image_input.Init(width, height, 1, format, usage, VK_IMAGE_TILING_OPTIMAL);
+    VkImageView view_input = image_input.targetView(format);
+
+    // Create render pass with a subpass that has input attachment.
+    vk_testing::RenderPass render_pass;
+    {
+        VkAttachmentDescription input_attachment = {};
+        input_attachment.format = format;
+        input_attachment.samples = VK_SAMPLE_COUNT_1_BIT;
+        input_attachment.loadOp = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
+        input_attachment.storeOp = VK_ATTACHMENT_STORE_OP_DONT_CARE;
+        input_attachment.stencilLoadOp = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
+        input_attachment.stencilStoreOp = VK_ATTACHMENT_STORE_OP_DONT_CARE;
+        input_attachment.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+        input_attachment.finalLayout = VK_IMAGE_LAYOUT_GENERAL;
+
+        const VkAttachmentReference attachment_reference = {0, VK_IMAGE_LAYOUT_GENERAL};
+
+        VkSubpassDescription subpass = {};
+        subpass.pipelineBindPoint = VK_PIPELINE_BIND_POINT_GRAPHICS;
+        subpass.inputAttachmentCount = 1;
+        subpass.pInputAttachments = &attachment_reference;
+
+        auto rpci = LvlInitStruct<VkRenderPassCreateInfo>();
+        rpci.attachmentCount = 1;
+        rpci.pAttachments = &input_attachment;
+        rpci.subpassCount = 1;
+        rpci.pSubpasses = &subpass;
+
+        render_pass.init(*m_device, rpci);
+        ASSERT_TRUE(render_pass.initialized());
+    }
+
+    auto fbci = LvlInitStruct<VkFramebufferCreateInfo>();
+    fbci.renderPass = render_pass.handle();
+    fbci.attachmentCount = 1;
+    fbci.pAttachments = &view_input;
+    fbci.width = width;
+    fbci.height = height;
+    fbci.layers = 1;
+    vk_testing::Framebuffer fb(*m_device, fbci);
+    ASSERT_TRUE(fb.initialized());
+
+    char const *fsSource = R"glsl(
+        #version 450
+        layout(input_attachment_index=0, set=0, binding=0) uniform subpassInput x;
+        void main() {
+        vec4 color = subpassLoad(x);
+        }
+    )glsl";
+    VkShaderObj vs(this, bindStateVertShaderText, VK_SHADER_STAGE_VERTEX_BIT);
+    VkShaderObj fs(this, fsSource, VK_SHADER_STAGE_FRAGMENT_BIT);
+
+    const VkDescriptorSetLayoutBinding binding = {0, VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT, 1, VK_SHADER_STAGE_FRAGMENT_BIT, nullptr};
+    OneOffDescriptorSet descriptor_set(m_device, {binding});
+    descriptor_set.WriteDescriptorImageInfo(0, view_input, VK_NULL_HANDLE, VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT,
+                                            VK_IMAGE_LAYOUT_GENERAL);
+    descriptor_set.UpdateDescriptorSets();
+    const VkDescriptorSetLayoutObj ds_layout_unused(m_device, {binding});
+    const VkPipelineLayoutObj pipeline_layout(m_device, {&descriptor_set.layout_, &ds_layout_unused});
+
+    VkPipelineObj pipe(m_device);
+    pipe.SetViewport(m_viewports);
+    pipe.SetScissor(m_scissors);
+    pipe.AddShader(&vs);
+    pipe.AddShader(&fs);
+    ASSERT_VK_SUCCESS(pipe.CreateVKPipeline(pipeline_layout.handle(), render_pass.handle()));
+
+    m_commandBuffer->begin();
+    m_renderPassBeginInfo.renderPass = render_pass.handle();
+    m_renderPassBeginInfo.framebuffer = fb.handle();
+    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
+    vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.handle());
+    vk::CmdBindDescriptorSets(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline_layout.handle(), 0, 1,
+                              &descriptor_set.set_, 0, nullptr);
+
+    // This draw command will likely produce a crash in case of a regression.
+    vk::CmdDraw(m_commandBuffer->handle(), 1, 0, 0, 0);
+
+    m_commandBuffer->EndRenderPass();
+    m_commandBuffer->end();
+}
+
+TEST_F(VkPositiveLayerTest, UpdateDescritorSetsNoLongerInUse) {
+    TEST_DESCRIPTION("Use descriptor in the draw call and then update descriptor when it is no longer in use");
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    ASSERT_NO_FATAL_FAILURE(InitState(nullptr, nullptr, VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT));
+    ASSERT_NO_FATAL_FAILURE(InitViewport());
+    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
+
+    VkCommandBufferObj cb0(m_device, m_commandPool);
+    VkCommandBufferObj cb1(m_device, m_commandPool);
+
+    for (int mode = 0; mode < 2; mode++) {
+        const bool use_single_command_buffer = (mode == 0);
+        //
+        // Create resources.
+        //
+        const VkDescriptorPoolSize pool_size = {VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 2};
+        auto descriptor_pool_ci = LvlInitStruct<VkDescriptorPoolCreateInfo>();
+        descriptor_pool_ci.flags = 0;
+        descriptor_pool_ci.maxSets = 2;
+        descriptor_pool_ci.poolSizeCount = 1;
+        descriptor_pool_ci.pPoolSizes = &pool_size;
+        vk_testing::DescriptorPool pool(*m_device, descriptor_pool_ci);
+
+        const VkDescriptorSetLayoutBinding binding = {0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_FRAGMENT_BIT,
+                                                      nullptr};
+        auto set_layout_ci = LvlInitStruct<VkDescriptorSetLayoutCreateInfo>();
+        set_layout_ci.flags = 0;
+        set_layout_ci.bindingCount = 1;
+        set_layout_ci.pBindings = &binding;
+        vk_testing::DescriptorSetLayout set_layout(*m_device, set_layout_ci);
+
+        VkDescriptorSet set_A = VK_NULL_HANDLE;
+        VkDescriptorSet set_B = VK_NULL_HANDLE;
+        {
+            const VkDescriptorSetLayout set_layouts[2] = {set_layout, set_layout};
+            auto set_alloc_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
+            set_alloc_info.descriptorPool = pool;
+            set_alloc_info.descriptorSetCount = 2;
+            set_alloc_info.pSetLayouts = set_layouts;
+            VkDescriptorSet sets[2] = {};
+            ASSERT_VK_SUCCESS(vk::AllocateDescriptorSets(device(), &set_alloc_info, sets));
+            set_A = sets[0];
+            set_B = sets[1];
+        }
+
+        auto buffer_ci = LvlInitStruct<VkBufferCreateInfo>();
+        buffer_ci.size = 1024;
+        buffer_ci.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
+        vk_testing::Buffer buffer(*m_device, buffer_ci, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+
+        VkShaderObj vs(this, bindStateVertShaderText, VK_SHADER_STAGE_VERTEX_BIT);
+        VkShaderObj fs(this, bindStateFragUniformShaderText, VK_SHADER_STAGE_FRAGMENT_BIT);
+
+        auto pipeline_layout_ci = LvlInitStruct<VkPipelineLayoutCreateInfo>();
+        pipeline_layout_ci.setLayoutCount = 1;
+        pipeline_layout_ci.pSetLayouts = &set_layout.handle();
+        vk_testing::PipelineLayout pipeline_layout(*m_device, pipeline_layout_ci);
+
+        VkPipelineObj pipe(m_device);
+        pipe.SetViewport(m_viewports);
+        pipe.SetScissor(m_scissors);
+        pipe.AddDefaultColorAttachment();
+        pipe.AddShader(&vs);
+        pipe.AddShader(&fs);
+        pipe.CreateVKPipeline(pipeline_layout.handle(), m_renderPass);
+
+        auto update_set = [this](VkDescriptorSet set, VkBuffer buffer) {
+            VkDescriptorBufferInfo buffer_info = {};
+            buffer_info.buffer = buffer;
+            buffer_info.offset = 0;
+            buffer_info.range = VK_WHOLE_SIZE;
+
+            auto descriptor_write = LvlInitStruct<VkWriteDescriptorSet>();
+            descriptor_write.dstSet = set;
+            descriptor_write.descriptorCount = 1;
+            descriptor_write.descriptorType = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER;
+            descriptor_write.pBufferInfo = &buffer_info;
+            vk::UpdateDescriptorSets(device(), 1, &descriptor_write, 0, nullptr);
+        };
+
+        //
+        // Test scenario.
+        //
+        update_set(set_A, buffer);
+        update_set(set_B, buffer);
+
+        // Bind set A to a command buffer and submit the command buffer;
+        {
+            auto &cb = use_single_command_buffer ? *m_commandBuffer : cb0;
+            cb.begin();
+            vk::CmdBindDescriptorSets(cb, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline_layout.handle(), 0, 1, &set_A, 0, nullptr);
+            vk::CmdBindPipeline(cb, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.handle());
+            cb.BeginRenderPass(m_renderPassBeginInfo);
+            vk::CmdDraw(cb, 0, 0, 0, 0);
+            vk::CmdEndRenderPass(cb);
+            cb.end();
+            auto submit_info = LvlInitStruct<VkSubmitInfo>();
+            submit_info.commandBufferCount = 1;
+            submit_info.pCommandBuffers = &cb.handle();
+            ASSERT_VK_SUCCESS(vk::QueueSubmit(m_device->m_queue, 1, &submit_info, VK_NULL_HANDLE));
+        }
+
+        // Wait for the queue. After this set A should be no longer in use.
+        ASSERT_VK_SUCCESS(vk::QueueWaitIdle(m_device->m_queue));
+
+        // Bind set B to a command buffer and submit the command buffer;
+        {
+            auto &cb = use_single_command_buffer ? *m_commandBuffer : cb1;
+            cb.begin();
+            vk::CmdBindDescriptorSets(cb, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline_layout.handle(), 0, 1, &set_B, 0, nullptr);
+            vk::CmdBindPipeline(cb, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.handle());
+            cb.BeginRenderPass(m_renderPassBeginInfo);
+            vk::CmdDraw(cb, 0, 0, 0, 0);
+            vk::CmdEndRenderPass(cb);
+            cb.end();
+            auto submit_info = LvlInitStruct<VkSubmitInfo>();
+            submit_info.commandBufferCount = 1;
+            submit_info.pCommandBuffers = &cb.handle();
+            ASSERT_VK_SUCCESS(vk::QueueSubmit(m_device->m_queue, 1, &submit_info, VK_NULL_HANDLE));
+        }
+
+        // Update set A. It should not cause VU 03047 error.
+        vk_testing::Buffer buffer2(*m_device, buffer_ci, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+        update_set(set_A, buffer2);
+
+        ASSERT_VK_SUCCESS(vk::QueueWaitIdle(m_device->m_queue));
+    }
+}
